@@ -44,7 +44,8 @@ static void usage(void)
 		"Usage: ip rule { add | del } SELECTOR ACTION\n"
 		"       ip rule { flush | save | restore }\n"
 		"       ip rule [ list [ SELECTOR ]]\n"
-		"SELECTOR := [ not ] [ from PREFIX ] [ to PREFIX ] [ tos TOS ] [ fwmark FWMARK[/MASK] ]\n"
+		"SELECTOR := [ not ] [ from PREFIX ] [ to PREFIX ] [ tos TOS ]\n"
+		"            [ fwmark FWMARK[/MASK] ]\n"
 		"            [ iif STRING ] [ oif STRING ] [ pref NUMBER ] [ l3mdev ]\n"
 		"            [ uidrange NUMBER-NUMBER ]\n"
 		"            [ ipproto PROTOCOL ]\n"
